$EventSystem::Version = 1.74;

function reloadEventingServer()
{
	exec("add-ons/system_eventing/server.cs");
}

function canSubmitEvents(%client)
{
	if(!isObject(%client.wrenchBrick))
		return false;
	return getTrustLevel(%client.wrenchBrick,%client) >= 2;
}

function servercmdeventsystemversion(%client,%version)
{
	%client.hasSystemEventing = true;
	%client.systemEventingVersion = %version;
	commandtoclient(%client,'eventsystemversion',$EventSystem::Version);
	if(%version == $EventSystem::Version)
		messageclient(%client,'',"\c3You have the same version of System_Eventing as the server.");
	if(%version < $EventSystem::Version)
		messageclient(%client,'',"\c3The server has a \c2newer\c3 version of System_Eventing than you.");
	if(%version > $EventSystem::Version)
		messageclient(%client,'',"\c3The server has a \c2older\c3 version of System_Eventing than you.");
}

function servercmdaddeventindent(%client,%a,%indent)
{
	if(!canSubmitEvents(%client))
		return;
	if(%indent < 0)
		return;
	if(!isObject(%client.wrenchBrick))
		messageclient(%client,'',"No wrench brick!");
	else
	{
		%brick = %client.wrenchBrick;
		%brick.eventIndent[%a] = %indent;
	}	
}

function servercmdaddeventcolor(%client,%a,%color)
{
	if(!canSubmitEvents(%client))
		return;
	if(%color < 0)
		%color = 0;
	if(!isObject(%client.wrenchBrick))
		messageclient(%client,'',"No wrench brick!");
	else
	{
		%brick = %client.wrenchBrick;
		%brick.eventColor[%a] = %color;
	}	
}

function getNameIndex(%brickGroup,%name)
{
	for(%a=0;%a<%brickGroup.NTNameCount;%a++)
		if(%name $= %brickGroup.NTName[%a])
			return %a;
	return -1;
}

function servercmdrequestNewEvents(%client)
{
	if(!isObject(%client.wrenchBrick))
		messageclient(%client,'',"No wrench brick!");
	else
	{
		%brick = %client.wrenchBrick;
		for(%a=0;%a<%brick.numEvents;%a++)
		{
			commandtoclient(%client,'addEvent',%brick.eventEnabled[%a],%brick.eventDelay[%a],%brick.eventInputIdx[%a],%brick.eventTargetIdx[%a],%brick.eventOutputIdx[%a],%a);
			if(strlen($InputEvent_CheckfxDTSBrick_[%brick.eventInputIdx[%a]]) > 0)
				commandtoclient(%client,'setInputParam',%a,%brick.eventInputParameter[%a]);
			commandtoclient(%client,'addindenttoevent',%a,%brick.eventIndent[%a]);
			if(!(%brick.eventColor[%a] > 0))
				%brick.eventColor[%a] = 0;
			commandtoclient(%client,'addcolortoevent',%a,%brick.eventColor[%a]);
			if(%brick.eventTargetIdx[%a] == -1 && strlen(%brick.eventNT[%a]) > 0)
			{
				if(%brick.getGroup().bl_id == %client.bl_id)
					commandtoclient(%client,'addnamedbricktoevent',%a,%brick.eventNT[%a]);
				else
					commandtoclient(%client,'addnamedbricktoevent',%a,getNameIndex(%brick.getGroup(),%brick.eventNT[%a]));
			}
			for(%b=1;strlen(%brick.eventOutputParameter[%a,%b]) > 0;%b++)
				commandtoclient(%client,'addparamtoevent',%a,%b-1,%brick.eventOutputParameter[%a,%b]);
		}
	}
}

function registerInputParameter(%event,%param,%function)
{
	%id = -1;
	for(%a=0;%a<$InputEvent_CountfxDTSBrick;%a++)
	{
		if($InputEvent_NamefxDTSBrick_[%a] $= %event)
		{
			%id = %a;
			break;
		}
	}
	if(%id == -1)
	{
		error("Could not register input parameter: " @ %event @ " not found");
		return;
	}

	if(strlen($InputEvent_CheckfxDTSBrick_[%id]) > 0 || strlen($InputEvent_ParameterfxDTSBrick_[%id]) > 0)
		echo("Overwriting previously registered input parameter for: " @ %event);
	
	$InputEvent_ParameterfxDTSBrick_[%id] = %param;
	$InputEvent_CheckfxDTSBrick_[%id] = %function;
}

function servercmdrequestInputParams(%client)
{
	for(%a=0;%a<$InputEvent_CountfxDTSBrick;%a++)
		if(strlen($InputEvent_ParameterfxDTSBrick_[%a]) > 0)
			commandtoclient(%client,'addInputParam',%a,$InputEvent_ParameterfxDTSBrick_[%a]);
}

function servercmdsetInputParam(%client,%id,%param)
{
	if(!canSubmitEvents(%client))
		return;
	if(!isObject(%client.wrenchBrick))
		messageclient(%client,'',"No wrench brick!");
	else
	{
		%brick = %client.wrenchBrick;
		%brick.eventInputParameter[%id] = %param;
	}	
}

package inputEventParameters
{
	function servercmdcancelallevents(%client)
	{
		if(!%client.isAdmin && !%client.isSuperAdmin)
		{
			messageclient(%client,'',"You have to be admin to do that!");
			return;
		}
		globalquota.cancelevents();
		for(%a=0;%a<clientGroup.getCount();%a++)
			getQuotaObjectFromClient(clientGroup.getObject(%a)).cancelEvents();
		parent::servercmdcancelallevents(%client);
	}

	function servercmdcancelevents(%client)
	{
		getQuotaObjectFromClient(%client).cancelEvents();
		Parent::servercmdcancelevents(%client);
	}

	function SimObject::processInputEvent(%brick,%event,%client,%specialObject)
	{
		if(!isObject(%brick))
			error("Brick for processInputEvent does not exist.");
		for(%a=0;%a<%brick.numEvents;%a++)
		{
			if(%brick.eventInput[%a] !$= %event)
				continue;
			if(!%brick.eventEnabled[%a])
				continue;

			%inputID = %brick.eventInputIdx[%a];
			%targetID = %brick.eventTargetIdx[%a];
			%outputID = %brick.eventOutputIdx[%a];
			%outputFunc = %brick.eventOutput[%a];
			if(strpos(%brick.outputFunc,"relay") != -1 && %brick.eventDelay[%a] < 33)
				%brick.eventDelay[%a] = 33;
			%targetClass = inputEvent_GetTargetClass(%brick.getClassName(),%inputID,%targetID);
			%targetObj[0] = $InputTarget_[%brick.eventTarget[%a]];
			%targetCount = 1;
			if(%brick.eventTarget[%a] == -1)
			{
				%targetName = %brick.eventNT[%a];
				%targetCount = %brick.getGroup().NTObjectCount[%targetName];
				for(%b=0;%b<%targetcount;%b++)
					%targetObj[%b] = %brick.getgroup().ntobject[%targetname,%b];
				%targetClass = "fxDtsBrick";
			}

			%shouldContinue = true;
			if(strlen($InputEvent_ParameterfxDTSBrick_[%inputID]) > 0)
			{
				%shouldContinue = false;
				%param = "\"" @ %brick.eventInputParameter[%a] @ "\"";	
				if(!isObject(%specialObject))
					eval("%shouldContinue = " @ $InputEvent_CheckfxDTSBrick_[%inputID] @ "(" @ %brick @ ",\"" @ %client @ "\",\"" @ %targetObj[0] @ "\"," @ %param @ ");");
				else
					eval("%shouldContinue = " @ $InputEvent_CheckfxDTSBrick_[%inputID] @ "(" @ %brick @ ",\"" @ %client @ "\",\"" @ %targetObj[0] @ "\"," @ %param @ "," @ %specialObject @ ");");
			}
			if(%shouldContinue)
			{
				%quotaObject = -1;
				if(isObject(%client))
					%quotaObject = getQuotaObjectFromClient(%client);
				else if(isObject(%brick))
					%quotaObject = getQuotaObjectFromBrick(%brick);
				%numParams = outputEvent_GetnumParametersFromIdx(%targetClass,%outputID);
				for(%b=0;%b<%numParams;%b++)
					%param[%b] = %brick.eventOutputParameter[%a,%b+1];
				%appendClient = $OutputEvent_AppendClient[%targetClass,%outputID];
				for(%c=0;%c<%targetCount;%c++)
				{
					%evalLine = "$lastEventSch = " @ %targetObj[%c] @ ".schedule(" @ %brick.eventDelay[%a] @ "," @ %outputFunc @ ",";
					for(%b=0;%b<%numParams;%b++)
						%evalLine = %evalLine @ "\"" @ %param[%b] @ "\",";
					if(%appendClient && isObject(%client))
						%evalLine = %evalLine @ %client;
					if(getSubStr(%evalLine,strlen(%evalLine)-1,1) $= ",")
						%evalLine = getSubStr(%evalLine,0,strlen(%evalLine)-1);
					%evalLine = %evalLine @ ");";
					eval(%evalLine);
					if(isObject(%quotaObject))
						%quotaObject.addScheduledEvent($lastEventSch);
					if(%quotaObject != GlobalQuota.getID())
						GlobalQuota.addScheduledEvent($lastEventSch);
					if(isObject(%client))
						%client.addScheduledEvent($lastEventSch);
					%brick.addScheduledEvent($lastEventSch);
				}
			}	
		}
	}
};
activatepackage(inputEventParameters);
$inputEventParametersPackage = true;

function onActivateCheck(%brick,%client,%target,%param,%specialObject)
{
	if(strlen(%param) < 1 || %param == -1)
		return true;	
	else 
		return %param $= %client.name;
}

registerInputParameter("onActivate","string","onActivateCheck");

function onProjectileHitCheck(%brick,%client,%target,%param,%specialObject)
{
	if(strlen(%param) < 1 || %param == -1)
		return true;	
	else 
		return $InputTarget_["Projectile"].dataBlock.getID() == %param;
}

registerInputParameter("onProjectileHit","datablock\tProjectileData","onProjectileHitCheck");

function servercmdmegacheck(%client)
{
	messageclient(%client,'',"\c3This server has system_MEGAevents version: \c6" @ $EventSystem::Version);
	for(%a=0;%a<clientGroup.getCount();%a++)
		if(clientGroup.getObject(%a).hasSystemEventing)
			messageclient(%client,'',"\c6" @ clientGroup.getObject(%a).name @ "\c3 has system_MEGAevents version: \c6" @ clientGroup.getobject(%a).systemEventingVersion);
}

function servercmdtoggleeventpackage(%client)
{
	if(%client.isSuperAdmin)
	{
		if($inputEventParametersPackage)
		{
			deactivatepackage(inputEventParameters);
			$inputEventParametersPackage = false;
			messageclient(%client,'',"\c3You turned the system_MEGAevents package \c6off\c3.");
		}
		else
		{
			activatepackage(inputEventParameters);
			$inputEventParametersPackage = true;
			messageclient(%client,'',"\c3You turned the system_MEGAevents package \c6on\c3.");
		}
	}
}
